unit ufrmMainFactory;

interface

uses
  Windows
  , Messages
  , SysUtils
  , Classes
  , Graphics
  , Controls
  , Forms
  , Dialogs
  , uToyFactory
  , StdCtrls, ExtCtrls
  ;

type
  TfrmMainFactory = class(TForm)
    lstToys: TListBox;
    cmdSet1: TButton;
    cmdSet2: TButton;
    cmdSet3: TButton;
    cmdPlay: TButton;
    cmdTidy: TButton;
    cmdGive: TButton;
    rgrpToyTypes: TRadioGroup;
    cmdClose: TButton;
    procedure cmdSet1Click(Sender: TObject);
    procedure cmdSet2Click(Sender: TObject);
    procedure cmdSet3Click(Sender: TObject);
    procedure cmdPlayClick(Sender: TObject);
    procedure cmdTidyClick(Sender: TObject);
    procedure cmdGiveClick(Sender: TObject);
    procedure rgrpToyTypesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdCloseClick(Sender: TObject);
  private
    { Private declarations }
    FFactory: TToyFactory;
    FToyType: integer;

    procedure SetFactory(Value: TToyFactory);

  public
    { Public declarations }
    property Factory: TToyFactory
      write SetFactory;
  end;

var
  frmMainFactory: TfrmMainFactory;

implementation

uses
  uToy
  , uBricksFactory
  , uDollsFactory
  ;

{$R *.DFM}

procedure TfrmMainFactory.cmdSet1Click(Sender: TObject);
var
  nItem: integer;
begin
  // Get set no. 1
  if Assigned(FFactory) then
  begin
    nItem := lstToys.Items.AddObject('', FFactory.GetSet1);
    lstToys.Items.Strings[nItem] := TToy(lstToys.Items.Objects[nItem]).Name;
  end;
end;

procedure TfrmMainFactory.cmdSet2Click(Sender: TObject);
var
  nItem: integer;
begin
  // Get set no. 2
  if Assigned(FFactory) then
  begin
    nItem := lstToys.Items.AddObject('', FFactory.GetSet2);
    lstToys.Items.Strings[nItem] := TToy(lstToys.Items.Objects[nItem]).Name;
  end;
end;

procedure TfrmMainFactory.cmdSet3Click(Sender: TObject);
var
  nItem: integer;
begin
  // Get set no. 3
  if Assigned(FFactory) then
  begin
    nItem := lstToys.Items.AddObject('', FFactory.GetSet3);
    lstToys.Items.Strings[nItem] := TToy(lstToys.Items.Objects[nItem]).Name;
  end;
end;

procedure TfrmMainFactory.cmdPlayClick(Sender: TObject);
begin
  // Play with the selected toy set
  if (lstToys.ItemIndex > -1) then
    TToy(lstToys.Items.Objects[lstToys.ItemIndex]).PlayWithMe;
end;

procedure TfrmMainFactory.cmdTidyClick(Sender: TObject);
begin
  // Tidy the selected toy set away
  if (lstToys.ItemIndex > -1) then
    TToy(lstToys.Items.Objects[lstToys.ItemIndex]).TidyMeAway;
end;

procedure TfrmMainFactory.cmdGiveClick(Sender: TObject);
begin
  // Give the selected toy set away to another child
  if (lstToys.ItemIndex > -1) then
  begin
    TToy(lstToys.Items.Objects[lstToys.ItemIndex]).Free;
    lstToys.Items.Delete(lstToys.ItemIndex);
  end;
end;

procedure TfrmMainFactory.rgrpToyTypesClick(Sender: TObject);
begin
  try
    case rgrpToyTypes.ItemIndex of
      0: Factory := TBricksFactory.Create;
      1: Factory := TDollsFactory.Create;
    end;
    FToyType := rgrpToyTypes.ItemIndex;
  except
    rgrpToyTypes.ItemIndex := FToyType;
    raise;
  end;
end;

procedure TfrmMainFactory.SetFactory(Value: TToyFactory);
begin
  if not Assigned(FFactory) or (Value.ClassType <> FFactory.ClassType) then
  begin
    if (lstToys.Items.Count <> 0) then
    begin
      Value.Free;
      raise Exception.Create('You must give away all your toys before you can have one of a different type');
    end;
    if Assigned(FFactory) then
      FFactory.Free;
    FFactory := Value;
  end
  else
  begin
    Value.Free;
  end;
end;

procedure TfrmMainFactory.FormCreate(Sender: TObject);
begin
  FToyType := -1;
end;

procedure TfrmMainFactory.cmdCloseClick(Sender: TObject);
begin
  Self.Close;
end;

end.
