unit uBricksFactory;

interface

uses
  uToyFactory
  , uToy
  , classes                         // for RegisterClass & TList
  ;

type

  TBricksFactory = class(TToyFactory)
  private

    FFreeList: TList;
    FNextToy: integer;

  protected

    function Borrow: TToy; override;

  public

    procedure Return(Toy: TToy); override;

    constructor Create; override;
    destructor Destroy; override;

    function GetSet1: TToy; override;
    function GetSet2: TToy; override;
    function GetSet3: TToy; override;

  published

  end;

implementation

uses
  uBricks
  , sysutils                            // for Exception
  ;

{ TBricksFactory }

const
  nMaxToys: integer = 5;

function TBricksFactory.Borrow: TToy;
begin
  if FNextToy > -1 then
  begin
    Result := TToy(FFreeList.Items[FNextToy]);
    FFreeList.Items[FNextToy] := nil;           // not essential to do this
    Dec(FNextToy);
  end
  else
  begin
    raise Exception.Create('There are no more bricks in the library');
  end;
end;

constructor TBricksFactory.Create;
var
  nToy: integer;
begin
  FFreeList := TList.Create;
  for nToy := 0 to Pred(nMaxToys) do
    FFreeList.Add(TBricks.Create);
  FNextToy := Pred(FFreeList.Count);
end;

destructor TBricksFactory.Destroy;
var
  nToy: integer;
begin
  // Assume that no toys are still out on loan
  for nToy := 0 to FNextToy do
    TBricks(FFreeList.Items[nToy]).Free;
  FFreeList.Free;
  inherited;
end;

function TBricksFactory.GetSet1: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 1;
end;

function TBricksFactory.GetSet2: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 2;
end;

function TBricksFactory.GetSet3: TToy;
begin
  Result := Self.Borrow;
  Result.SetNo := 3;
end;

procedure TBricksFactory.Return(Toy: TToy);
begin
  if (FNextToy < Pred(FFreeList.Count)) then
  begin
    Inc(FNextToy);
    FFreeList.Items[FNextToy] := Toy;
  end
  else
  begin
    // This should never happen
    raise Exception.Create('No room in the library to return this toy');
  end;
end;

initialization
  RegisterClass(TBricksFactory);
  RegisterFactory('Bricks', 'TBricksFactory');

end.
